@@ -52,6 +52,8 @@
  * result of user feedback. This library is platform dependent and therefore it
  * has implementations for both dart2js and the Dart VM. Both are under
  * development and may not support all operations yet.
+ *
+ * {@category VM}
  */
 library dart.mirrors;
 
@@ -173,7 +175,7 @@ external ClassMirror reflectClass(Type key);
  *
  * Optionally takes a list of [typeArguments] for generic classes. If the list
  * is provided, then the [key] must be a generic class type, and the number of
- * the provided type arguments must be equal to the number of type variables 
+ * the provided type arguments must be equal to the number of type variables
  * declared by the class.
  *
  * Note that since one cannot obtain a [Type] object from another isolate, this
@@ -217,7 +219,24 @@ abstract class IsolateMirror implements Mirror {
    * 2. the isolate being reflected by this mirror is the same isolate being
    *    reflected by [other].
    */
-  bool operator == (other);
+  bool operator ==(other);
+
+  /**
+   * Loads the library at the given uri into this isolate.
+   *
+   * WARNING: You are strongly encouraged to use Isolate.spawnUri instead when
+   * possible. IsolateMirror.loadUri should only be used when synchronous
+   * communication or shared state with dynamically loaded code is needed.
+   *
+   * If a library with the same canonicalized uri has already been loaded,
+   * the existing library will be returned. (The isolate will not load a new
+   * copy of the library.)
+   *
+   * This behavior is similar to the behavior of an import statement that
+   * appears in the root library, except that the import scope of the root
+   * library is not changed.
+   */
+  Future<LibraryMirror> loadUri(Uri uri);
 }
 
 /**
@@ -359,7 +378,6 @@ abstract class DeclarationMirror implements Mirror {
  * See [InstanceMirror], [ClassMirror], and [LibraryMirror].
  */
 abstract class ObjectMirror implements Mirror {
-
   /**
    * Invokes the named function and returns a mirror on the result.
    *
@@ -385,9 +403,8 @@ abstract class ObjectMirror implements Mirror {
    * TODO(turnidge): Handle ambiguous names.
    * TODO(turnidge): Handle optional & named arguments.
    */
-  InstanceMirror invoke(Symbol memberName,
-                        List positionalArguments,
-                        [Map<Symbol,dynamic> namedArguments]);
+  InstanceMirror invoke(Symbol memberName, List positionalArguments,
+      [Map<Symbol, dynamic> namedArguments]);
 
   /**
    * Invokes a getter and returns a mirror on the result.
@@ -454,7 +471,7 @@ abstract class ObjectMirror implements Mirror {
   InstanceMirror setField(Symbol fieldName, Object value);
 
   /**
-   * Performs [invocation] on [reflectee].
+   * Performs [invocation] on the reflectee of this [ObjectMirror].
    *
    * Equivalent to
    *
@@ -462,7 +479,7 @@ abstract class ObjectMirror implements Mirror {
    *       return this.getField(invocation.memberName).reflectee;
    *     } else if (invocation.isSetter) {
    *       return this.setField(invocation.memberName,
-   *                            invocation.positionArguments[0]).reflectee;
+   *                            invocation.positionalArguments[0]).reflectee;
    *     } else {
    *       return this.invoke(invocation.memberName,
    *                          invocation.positionalArguments,
@@ -526,7 +543,7 @@ abstract class InstanceMirror implements ObjectMirror {
    *    b. the remote objects reflected by this mirror and by [other] are
    *    identical.
    */
-  bool operator == (other);
+  bool operator ==(other);
 }
 
 /**
@@ -579,7 +596,7 @@ abstract class ClosureMirror implements InstanceMirror {
    * method throws *e*.
    */
   InstanceMirror apply(List positionalArguments,
-                       [Map<Symbol, dynamic> namedArguments]);
+      [Map<Symbol, dynamic> namedArguments]);
 }
 
 /**
@@ -736,7 +753,6 @@ abstract class TypeMirror implements DeclarationMirror {
    */
   TypeMirror get originalDeclaration;
 
-
   /**
    * Checks the subtype relationship, denoted by `<:` in the language
    * specification.
@@ -818,13 +834,12 @@ abstract class ClassMirror implements TypeMirror, ObjectMirror {
    */
   Map<Symbol, MethodMirror> get staticMembers;
 
-
   /**
    * The mixin of this class.
    *
    * If this class is the result of a mixin application of the form S with M,
    * returns a class mirror on M. Otherwise returns a class mirror on
-   * [reflectee].
+   * the reflectee.
    */
   ClassMirror get mixin;
 
@@ -861,9 +876,8 @@ abstract class ClassMirror implements TypeMirror, ObjectMirror {
    * * If evaluating the expression throws an exception *e* (that it does not
    *   catch), this method throws *e*.
    */
-  InstanceMirror newInstance(Symbol constructorName,
-                             List positionalArguments,
-                             [Map<Symbol,dynamic> namedArguments]);
+  InstanceMirror newInstance(Symbol constructorName, List positionalArguments,
+      [Map<Symbol, dynamic> namedArguments]);
 
   /**
    * Whether this mirror is equal to [other].
@@ -876,7 +890,7 @@ abstract class ClassMirror implements TypeMirror, ObjectMirror {
    * Note that if the reflected class is an invocation of a generic class, 2.
    * implies that the reflected class and [other] have equal type arguments.
    */
-  bool operator == (other);
+  bool operator ==(other);
 
   /**
    * Returns whether the class denoted by the receiver is a subclass of the
@@ -936,7 +950,7 @@ abstract class TypeVariableMirror extends TypeMirror {
    * 1. [other] is a mirror of the same kind, and
    * 2. [:simpleName == other.simpleName:] and [:owner == other.owner:].
    */
-  bool operator == (other);
+  bool operator ==(other);
 }
 
 /**
@@ -1056,6 +1070,11 @@ abstract class MethodMirror implements DeclarationMirror {
    */
   bool get isFactoryConstructor;
 
+  /**
+   * Is the reflectee an extension method?
+   */
+  bool get isExtensionMember;
+
   /**
    * Whether this mirror is equal to [other].
    *
@@ -1064,7 +1083,7 @@ abstract class MethodMirror implements DeclarationMirror {
    * 1. [other] is a mirror of the same kind, and
    * 2. [:simpleName == other.simpleName:] and [:owner == other.owner:].
    */
-  bool operator == (other);
+  bool operator ==(other);
 }
 
 /**
@@ -1097,6 +1116,11 @@ abstract class VariableMirror implements DeclarationMirror {
    */
   bool get isConst;
 
+  /**
+   * Is the reflectee an extension member?
+   */
+  bool get isExtensionMember;
+
   /**
    * Whether this mirror is equal to [other].
    *
@@ -1105,7 +1129,7 @@ abstract class VariableMirror implements DeclarationMirror {
    * 1. [other] is a mirror of the same kind, and
    * 2. [:simpleName == other.simpleName:] and [:owner == other.owner:].
    */
-  bool operator == (other);
+  bool operator ==(other);
 }
 
 /**
@@ -1244,6 +1268,7 @@ class Comment {
  * will ensure that the target `Bar` from the current library and from library
  * `foo` is available for reflection. See also [override].
  */
+@Deprecated("No longer has any effect. Will be removed in a later release.")
 class MirrorsUsed {
   // Note: the fields of this class are untyped.  This is because the most
   // convenient way to specify symbols today is using a single string. In
